# Set up landmarker image viewer.
# Places in the tcl top window.  Looks for object named viewer.

if {[info commands rtExMath] != "rtExMath"} {

if {$app_type == "exe"} { 
  source ViewerAppTkImageViewerInteractor.tcl
} else {
  source "${landmarker_home}ViewerAppTkImageViewerInteractor.tcl"
}

# Take window level parameters from viewer
proc InitializeWindowLevelInterface {infilename xs ys} {
   global viewer sliceNumber
   global pixmin pixmax
   global landmarker_home
   global deciFactor
   global app_type
   global threshGUI
   global l w

   # Get parameters from viewer
   set w [viewer GetColorWindow]
   set l [viewer GetColorLevel]
   set sliceNumber [viewer GetZSlice]
   set zMin [viewer GetWholeZMin]
   set zMax [viewer GetWholeZMax]

# Use header info for initial windox and level.
set pixrange [expr $pixmax - $pixmin]
set pixmid [expr ($pixmax + $pixmin) / 2.0]

set l $pixmid
set w [expr abs ($pixrange / 4.0) ]

puts stdout "w: $w"
puts stdout "l: $l"
puts stdout "zMin: $zMin"
puts stdout "zMax: $zMax"
puts stdout "sliceNumber: $sliceNumber"

set sliceNumber [expr $zMax / 2.0]

# Create the image viewer user interface.

destroy .top
toplevel .top -visual best

set filename "Image Viewer: [file tail $infilename]"
wm title .top $filename  

menu .top.menu -type menubar

menu .top.menu.file -tearoff 1
menu .top.menu.edit -tearoff 1
menu .top.menu.options -tearoff 1
menu .top.menu.open -tearoff 1
menu .top.menu.info -tearoff 1
menu .top.menu.help -tearoff 1

.top.menu add cascade -menu .top.menu.file -label "File"
.top.menu add cascade -menu .top.menu.edit -label "Edit"
.top.menu add cascade -menu .top.menu.options -label "Options"
.top.menu add cascade -menu .top.menu.open -label "Open"
.top.menu add cascade -menu .top.menu.info -label "Info"
.top.menu add cascade -menu .top.menu.help -label "Help"

.top.menu.file add command -label Exit -command CloseImageViewer -foreground red
##.top.menu.file add command -label "Snapshot" -command SaveSnapshotImageViewer
##.top.menu.file add command -label Exit -command ::vtk::cb_exit -foreground red
##.top.menu.options add check -label "Threshold/Isosurface" -command ThresholdImageGUI -variable threshGUI 
.top.menu.open add command -label "Threshold/Isosurface Menu" -command ThresholdImageGUI
set threshGUI 1 
.top.menu.options add check -label "Reverse Colors" -command SetInverseVideo -variable video

.top.menu.edit add command -label "Median Filter (3x3x3)" -command MedianFilter3x3
.top.menu.edit add command -label "Median Filter (5x5x5)" -command MedianFilter5x5
.top.menu.edit add command -label "Subtract Two Images" -command CreateImageSubtractionMenu


set source_file "OpenVolumeSlicer.tcl"
if {$app_type == "exe"} { 
  set source_file $source_file
  .top.menu.open add command -label "Volume Slicer" -command { source "OpenVolumeSlicer.tcl" }
  set source_file "OpenVolumeRenderWithUserInterface.tcl"
  set comm [catch {"source $source_file"} ]
} else {
  set source_file $landmarker_home$source_file
  .top.menu.open add command -label "Volume Slicer" -command { source "${landmarker_home}OpenVolumeSlicer.tcl" }
  set source_file "OpenVolumeRenderWithUserInterface.tcl"
  set comm [catch {"source $landmarker_home$source_file"} ]
}
##.top.menu.open add command -label "Volume Render" -command $comm 

##.top.menu.options add check -label "Window/Level Adjust" -command WindowLevelGUI -variable wlGUI
.top.menu.info add command -label "Image Dimensions" -command DisplayImageDimensions
.top.menu.info add command -label "Image Intensities" -command DisplayImageIntensities

.top configure -menu .top.menu

# The rendering widget
frame .top.f1
vtkTkImageViewerWidget .top.f1.v1 -width $xs -height $ys -iv viewer
pack .top.f1.v1 -padx 3 -pady 3 -side left -fill both -expand t
pack .top.f1 -fill both -expand t

   frame .top.slice
   label .top.slice.label -text " Slice # "
   scale .top.slice.scale -from $zMin -to $zMax -orient horizontal \
     -command SetSlice -variable sliceNumber -background #f00
   set pixrange [expr $pixmax - $pixmin]
   #puts stdout "pixrange: $pixrange"
   #scale .top.slice.level -from 1 -to $pixrange -orient horizontal \
   #  -command SetLevelBySlider -variable windo -background #f01

   
   frame .top.wl
   frame .top.wl.f1
   label .top.wl.f1.windowLabel -text "Window"
   #scale .top.wl.f1.window -from 1 -to [expr $w * 2]  -orient horizontal \
   #  -command SetWindow -variable window -background #f0f
   scale .top.wl.f1.window -from 1 -to $pixrange -orient horizontal \
     -command SetWindow -variable window -background #f0f
   frame .top.wl.f2
   label .top.wl.f2.levelLabel -text "  Level  "
   ##scale .top.wl.f2.level -from [expr $l - $w] -to [expr $l + $w] \
   ##  -orient horizontal -command SetLevel -background #f0f
   scale .top.wl.f2.level -from $pixmin -to $pixmax \
     -orient horizontal -command SetLevel -background #f0f


   # resolutions less than 1.0
   if {$w < 10} {
      set res [expr 0.05 * $w]
      .top.wl.f1.window configure -resolution $res -from $res -to [expr 2.0 * $w]
      .top.wl.f2.level configure -resolution $res \
	-from [expr 0.0 + $l - $w] -to [expr 0.0 + $l + $w] 
   }

   .top.wl.f1.window set $w
   .top.wl.f2.level set $l
   
   pack .top.slice -side top
   pack .top.slice.label .top.slice.scale -side left
   pack .top.wl.f1 .top.wl.f2 -side top

   pack .top.wl.f1.windowLabel .top.wl.f1.window -side left
   pack .top.wl.f2.levelLabel .top.wl.f2.level -side left
   pack .top.wl -side left
   ##pack forget .top.wl
   pack .top.wl -side top

   frame .top.thresh
   label .top.thresh.lab -text "INTENSITY THRESHOLDING"
   pack .top.thresh -side left
   pack .top.thresh.lab -side top
   label .top.thresh.lab2 -text "Threshold"
   if {$pixmin == $pixmax} {
      scale .top.thresh.value -from [expr $l - $w] -to [expr $l + $w] \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue $l
   } else {
       set pmin [expr $pixmin - 1.0]
       set pmax [expr $pixmax + 1.0]
       scale .top.thresh.value -from $pmin -to $pmax \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue [expr ($pixmax - $pixmin) / 2.0 ]
   }
   .top.thresh.value set $tvalue
   pack .top.thresh.lab2 .top.thresh.value -side left
   pack forget .top.thresh

   frame .top.iso
   button .top.iso.b1 -background #0f0 -text " Create Isosurface "  -command CreateIsosurface
   set source_file "OpenVolumeRender.tcl"
   #if {$app_type == "exe"} { 
   #   button .top.iso.volb -background #0ff -text " Volume Surface Render " -command "source $source_file"
   #} else {
   #   button .top.iso.volb -background #0ff -text " Volume Surface Render " -command "source $landmarker_home$source_file"
   #}
   #pack .top.iso.b1 .top.iso.volb
   pack .top.iso.b1
   pack forget .top.iso 
   
   frame .top.shrink
   scale .top.shrink.x -from 1 -to 10 -orient horizontal -variable XShrinkFactor -resolution 1 -length 100
   label .top.shrink.xtxt -text "Shrink Factor X"
   label .top.shrink.ytxt -text "Shrink Factor Y"
   label .top.shrink.ztxt -text "Shrink Factor Z"
   tk_optionMenu .top.shrink.interp menubuttonoptions "Interpolation Off" "Interpolation On"
   label .top.shrink.decitxt -text "Decimate"
   scale .top.shrink.y -from 1 -to 10 -orient horizontal -variable YShrinkFactor -resolution 1 -length 100
   scale .top.shrink.z -from 1 -to 10 -orient horizontal -variable ZShrinkFactor -resolution 1 -length 100 
   scale .top.shrink.deci -from 0.0 -to 1.0 -orient horizontal -variable deciFactor -resolution 0.1 -length 100 
   pack .top.shrink.decitxt .top.shrink.deci .top.shrink.interp .top.shrink.ztxt .top.shrink.z .top.shrink.ytxt .top.shrink.y .top.shrink.xtxt .top.shrink.x -side bottom -fill y
   pack forget .top.shrink -side top -fill y
   set XShrinkFactor 2
   set YShrinkFactor 2
   set ZShrinkFactor 1
   set deciFactor 0.0
   .top.shrink.x set $XShrinkFactor
   .top.shrink.y set $YShrinkFactor
   .top.shrink.z set $ZShrinkFactor
   .top.shrink.deci set $deciFactor

# annotation frames
#frame .top.f2
#frame .top.f2.f1
#frame .top.f2.f2
#frame .top.f2.f3
#frame .top.f2.f4

#label .top.f2.f1.label -relief sunken 
#label .top.f2.f2.label -relief sunken 
#label .top.f2.f3.label -relief sunken 
#label .top.f2.f4.label -relief sunken 

#pack .top.f2.f1.label -fill x 
#pack .top.f2.f2.label -fill x 
#pack .top.f2.f3.label -fill x 
#pack .top.f2.f4.label -fill x 

#pack .top.f2.f1 .top.f2.f2 .top.f2.f3 .top.f2.f4 -fill x -side left -expand t
#pack .top.f1 -fill both -expand t
#pack .top.f2 -fill x 

   BindTkImageViewer .top.f1.v1  
# tie labels to variables embedded in the Tk Widget
#.top.f2.f1.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 WindowLevelString]
#.top.f2.f2.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 PixelPositionString]
#.top.f2.f3.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 SliceString]

#.top.f2.f1.label configure -textvariable [GetWidgetVariable .top.f1.v1 WindowLevelString]
#.top.f2.f2.label configure -textvariable [GetWidgetVariable .top.f1.v1 PixelPositionString]
#.top.f2.f3.label configure -textvariable [GetWidgetVariable .top.f1.v1 SliceString]

}

proc MedianFilter3x3 {} {
   # Problem: can't seem to get data into CurrentVolume (for e.g. write to disk via File->Save)
   # Current version therefore simply writes to C:\tmp
   vtkImageMedian3D medianfilter
      medianfilter SetInput [CurrentVolume GetOutput]
      medianfilter SetKernelSize 3 3 3 

vtkImageFlip dum
      dum SetInput [medianfilter GetOutput]

vtkImageReslice reslice
  reslice SetInput [dum GetOutput]
##CurrentVolume SetInput [dum GetOutput]
viewer SetInputConnection [medianfilter GetOutputPort] 

   vtkImageWriter imwriter
      imwriter SetInput [dum GetOutput]
       imwriter SetFileDimensionality 3
    set outfilename "C:/tmp/medianfiltered3x3x3.img"
  imwriter SetFileName $outfilename
  puts stdout "Writing $outfilename ..."
  imwriter DebugOn
  imwriter Write
  puts stdout "Done writing file"
  imwriter Delete

      #PreviousVolume SetInput [medianfilter GetOutput]
      ##CurrentVolume SetInput [viewer GetInput]
      ##medianfiletr ReleaseDataFlagOff 
      ##CurrentVolume SetInput [medianfilter GetOutputPort]
      #viewer SetInput [CurrentVolume GetOutput]
      #viewer Render
}

proc MedianFilter5x5 {} {
   # Problem: can't seem to get data into CurrentVolume (for e.g. write to disk via File->Save)
   # Current version therefore simply writes to C:\tmp
   vtkImageMedian3D medianfilter
      medianfilter SetInput [CurrentVolume GetOutput]
      medianfilter SetKernelSize 5 5 5 

vtkImageFlip dum
      dum SetInput [medianfilter GetOutput]

vtkImageReslice reslice
  reslice SetInput [dum GetOutput]
##CurrentVolume SetInput [dum GetOutput]
viewer SetInputConnection [medianfilter GetOutputPort] 

   vtkImageWriter imwriter
      imwriter SetInput [dum GetOutput]
       imwriter SetFileDimensionality 3
    set outfilename "C:/tmp/medianfiltered5x5x5.img"
  imwriter SetFileName $outfilename
  puts stdout "Writing $outfilename ..."
  imwriter DebugOn
  imwriter Write
  puts stdout "Done writing file"
  imwriter Delete

      #PreviousVolume SetInput [medianfilter GetOutput]
      ##CurrentVolume SetInput [viewer GetInput]
      ##medianfiletr ReleaseDataFlagOff 
      ##CurrentVolume SetInput [medianfilter GetOutputPort]
      #viewer SetInput [CurrentVolume GetOutput]
      #viewer Render
}


proc ImageSubtraction {} {
   # Strategy: Create a copy of the current volume, then read a new volume, subtract the two
   
   global tmpdir currentdir
   global is_inputfile is_outputfile

   vtkImageReslice FirstVolume
     FirstVolume SetInput [CurrentVolume GetOutput]
     FirstVolume SetResliceTransform identityTransform 

   # Read new volume
   if { [string match *.hdr $is_inputfile] } {
            ReadAnalyzeImage $is_inputfile
   } elseif { [string match *.hea $is_inputfile] } {
            ReadAnalyzeImageASCII $is_inputfile
   } else {
            puts stdout "Can't read this file: $is_inputfile"
            return
   }
    
   vtkImageMathematics subtractor
         subtractor SetInput1 [FirstVolume GetOutput]
         subtractor SetInput2 [CurrentVolume GetOutput]
         subtractor SetOperationToSubtract
 
   vtkImageFlip dum
         dum SetInput [subtractor GetOutput]

   ##vtkImageReslice reslice
         ImageContainer SetInput [dum GetOutput]

   viewer SetInputConnection [subtractor GetOutputPort] 
   wm title .top "Image Viewer: Difference"

   subtractor Delete
   dum Delete
   FirstVolume Delete
  

}

proc CreateImageSubtractionMenu {} {
   global tmpdir currentdir
   global is_inputfile is_outputfile
   global currentdir
   global VolumeFileHasBeenRead
   global ImageFlipY
   global xpix ypix zpix xs ys zs
   global endian
   global pixmin pixmax
   global bitpix datatype hea_endian


   proc CloseImageSubtractionMenu {} {
      wm withdraw .is
      destroy .is
    }

   proc SaveImageSubtraction {} {
      global tmpdir currentdir
      global is_inputfile is_outputfile
      global VolumeFileHasBeenRead
      global ImageFlipY
      global xpix ypix zpix xs ys zs
      global endian
      global pixmin pixmax
      global bitpix datatype hea_endian
     
      # Cannot use WriteAnalyzeImageASCII as it is hardwired to write contents of CurrentVolume
      # (and I cant seem to get the subtraction image into CurrentVolume)
      #WriteAnalyzeImageASCII $is_outputfile


   puts stdout "ImageFlipY $ImageFlipY"
   ###set ImageFlipY 1
  if {$ImageFlipY == 1} {
    # Flip volume BACK in y (was flipped at read, to show as in Mvox)
    vtkImageFlip temp
      temp SetInput [ImageContainer GetOutput]
      temp SetFilteredAxis 1
  } else {
    vtkImageFlip temp
      temp SetInput [ImageContainer GetOutput]
  }
  puts stdout "Warning: ImageFlipY is set to $ImageFlipY"
  ###puts stdout "Warning: ImageFlipY was not taken into account when writing image!!!"

  vtkImageWriter imwriter

  imwriter SetInput [temp GetOutput]
  temp Delete
  ##imwriter SetInput [CurrentVolume GetOutput]

  imwriter SetFileDimensionality 3
  imwriter SetFileName $is_outputfile
  puts stdout "Writing $is_outputfile ..."
  imwriter DebugOn
  imwriter Write
  puts stdout "Done"
  imwriter Delete

  set hea_extension ".hea"
  set pos [string last "." $is_outputfile]
  set pos [expr $pos - 1]
  set nam [string range $is_outputfile 0 $pos]
  puts stdout $pos
  set heaoutfilename $nam$hea_extension
  puts stdout $heaoutfilename
  WriteHEAFile $heaoutfilename
  puts stdout "Done" 
  }

  set w .is
  toplevel $w
  wm title $w "Image Subtraction"
  set font {Helvetica 14}

  button $w.close -font $font -text  " Close "  -width 15 -foreground red -command CloseImageSubtractionMenu
  pack $w.close -side bottom -pady 2m
  label $w.l -font $font -justify center -text "1) Select image to subtract from current image."
  label $w.l2 -font $font -justify center -text "2) Press the subtract button and view result."
  label $w.l3 -font $font -justify center -text "3) Select Output Filename."
  label $w.l4 -font $font -justify center -text "4) Press Save button if you wish to save the result. "
  pack $w.l $w.l2 $w.l3 $w.l4 -side top -pady 8
  
  foreach i {input output} {
    set f [frame $w.$i]
    label $f.lab -text "Select a file as $i: " -anchor e
    entry $f.ent -width 60
    button $f.but -text "Browse ..." -command "fileDialog $w $f.ent $i"
    pack $f.lab -side left
    pack $f.ent -side left -expand yes -fill x
    pack $f.but -side left
    pack $f -fill x -padx 1c -pady 3
  }
 
  proc fileDialog {w ent operation} {
    global tmpdir currentdir
    global is_inputfile is_outputfile

    if { $operation == "output" } {
       set types {
	   {"Image File"		{.img}	}
	   {"All files"		*}
       }
    } else {
       set types {   
        {{Landmarker Image File (ASCII header)}                {.hea}        }    
        {{Analyze Image (use binary header)}                   {.hdr}        }
        {{All Files }                           *             }
      }
    }
    if {$operation == "input"} {
	set file [tk_getOpenFile -initialdir $currentdir -filetypes $types -parent $w]
      set is_inputfile $file
    } else {
	set file [tk_getSaveFile -filetypes $types -parent $w \
	    -initialdir $currentdir -defaultextension .vtk]
      set is_outputfile $file
    }
    if [string compare $file ""] {
	$ent delete 0 end
	$ent insert 0 $file
	$ent xview end
    }
  }

  button $w.b0 -font $font -text " Subtract " -foreground blue -width 20 -justify center \
                                  -command ImageSubtraction 
  button $w.b1 -font $font -text " Save " -foreground blue -width 20 -justify center \
                                  -command SaveImageSubtraction 
  pack $w.b0 $w.b1 -side top
}


proc CloseImageViewer {} {
   # Destroy the vtkImageViewer first, then the TkImageViewerWidget
   viewer Delete
   CurrentVolume Delete
   wm withdraw .top
   destroy .top
   vtkImageViewer viewer
}

proc WindowLevelGUI {} {
  global wlGUI
  if { $wlGUI } {
    pack .top.wl -side left
  } else {
    pack forget .top.wl
  }
}

#proc OpenVolumeSlicer {} {
#  source OpenVolumeSlicer.tcl
#}

proc ThresholdImageGUI_old {} {
  # Threshold the image by use of a slider.
  # Create isosurface when wanted.
  global threshGUI
  global tvalue
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  if { $threshGUI } {
    pack .top.thresh -side left
    pack .top.iso -side left
    pack .top.shrink -side top -fill y
    #viewer SetInput [thresh GetOutput]
    #set tvalue $tvalue
    ThresholdImage $tvalue
    viewer Render
  } else {
    pack forget .top.thresh
    pack forget .top.iso
    pack forget .top.shrink
    viewer SetInput [CurrentVolume GetOutput]
    viewer Render
  }
}

proc ThresholdImageGUI {} {
  global tvalue pixmax pixmin l w

    proc CloseIntensityThresholdingMenu {} {
      viewer SetInput [CurrentVolume GetOutput]
      viewer Render
      wm withdraw .th
      destroy .th
    }
    proc ThreshOFF {} {
      global thresh
      viewer SetInput [CurrentVolume GetOutput]
      viewer Render
    }
    proc ThreshON {} {
      global tvalue
      ThresholdImage $tvalue
      viewer SetColorWindow 1
      viewer SetColorLevel 0
      .top.wl.f1.window set 1
      .top.wl.f2.level set 0

      viewer Render
    }

    toplevel .th
  wm title .th "Intensity Thresholding"
  set font {Helvetica 14}

  button .th.close -font $font -text  " Close "  -width 15 -foreground red -command CloseIntensityThresholdingMenu
  pack .th.close -side bottom -pady 2m

  label .th.l -font $font -justify center -text "Intensity Threshold"
  pack .th.l -side top -pady 8

  if {$pixmin == $pixmax} {
      scale .th.value -from [expr $l - $w] -to [expr $l + $w] \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue $l
   } else {
       set pmin [expr $pixmin - 1.0]
       set pmax [expr $pixmax + 1.0]
       puts stdout "pmin pmax $pmin $pmax"
       scale .th.value -from $pmin -to $pmax \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue -resolution 0.1 -length 500 -label "Intensity Threshold"
      set tvalue [expr ($pixmax + $pixmin) / 2.0 ]
      puts stdout "tvalue $tvalue"
   }
   .th.value set $tvalue
   pack .th.value -pady 4

   frame .th.xx
   scale .th.xx.x -from 1 -to 10 -orient horizontal -variable XShrinkFactor -resolution 1 -length 100 
   label .th.xx.xtxt -text "Shrink Factor in X"
   pack .th.xx.x .th.xx.xtxt -side left -fill y

   frame .th.yy
   scale .th.yy.y -from 1 -to 10 -orient horizontal -variable YShrinkFactor -resolution 1 -length 100 
   label .th.yy.ytxt -text "Shrink Factor in Y"
   pack .th.yy.y .th.yy.ytxt -side left -fill y

   frame .th.zz
   scale .th.zz.z -from 1 -to 10 -orient horizontal -variable ZShrinkFactor -resolution 1 -length 100 
   label .th.zz.ztxt -text "Shrink Factor in Z"
   pack .th.zz.z .th.zz.ztxt -side left -fill y

   frame .th.dd
   scale .th.dd.deci -from 0.0 -to 1.0 -orient horizontal -variable deciFactor -resolution 0.1 -length 100 
   label .th.dd.decitxt -text "Decimate"
   pack .th.dd.deci .th.dd.decitxt -side left -fill y -pady 10

   frame .th.onoff
   button .th.onoff.on -text " Threshold ON "  -foreground green -width 15 -justify center -command ThreshON
   button .th.onoff.off -text " Threshold OFF "  -foreground blue -width 15 -justify center -command ThreshOFF
   pack .th.onoff.on .th.onoff.off -side left -expand 1 -pady 10

   pack .th.xx .th.yy .th.zz .th.dd .th.onoff -fill y

   button .th.b1 -font $font -background #0f0 -text " Create Isosurface "  -foreground blue -width 15 -justify center -command CreateIsosurface
   pack .th.b1 -side left -expand 1
      
   set XShrinkFactor 2
   set YShrinkFactor 2
   set ZShrinkFactor 1
   set deciFactor 0.0
   .th.xx.x set $XShrinkFactor
   .th.yy.y set $YShrinkFactor
   .th.zz.z set $ZShrinkFactor
   .th.dd.deci set $deciFactor





}


proc ThresholdImage {tvalue} {
  global thresh uvalue
  global oactorNum
  vtkImageThreshold thresh
  thresh SetInput [CurrentVolume GetOutput]
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  set uvalue [expr $l + $w]
  puts stdout "w l $w $l"
  viewer SetColorWindow 1
  viewer SetColorLevel 0
  #.top.wl.f1.window set 1
  #.top.wl.f2.level set 0

  thresh ThresholdByUpper $tvalue 
  thresh SetInValue $uvalue
  thresh SetOutValue 0
  viewer SetInput [thresh GetOutput]
  viewer Render
  thresh Delete
}


proc SetLevelBySlider {windo} {
  global viewer 
  viewer SetColorLevel $windo
  viewer Render
}
   
proc SetSlice { slice } {
   global sliceNumber viewer

   viewer SetZSlice $slice
   viewer Render
}

proc SetWindow window {
   global viewer video
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

proc SetLevel level {
   global viewer
   viewer SetColorLevel $level
   viewer Render
}

proc SetInverseVideo {} {
   global viewer video window
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

InitializeWindowLevelInterface $infilename $xs $ys

} else {
  viewer Render
}

proc DisplayImageDimensions {} {
  global xpix ypix zpix xs ys zs
  set mess "Pixel size: $xpix $ypix $zpix , Npix: $xs $ys $zs"
  notify_message $mess
}
proc DisplayImageIntensities {} {
  global pixmin pixmax
  set mess "Intensity range min max: $pixmin , $pixmax"
  notify_message $mess
}

# Procedure to save snapshot as a file.
# Viker ikke
proc SaveSnapshotImageViewer {} {
    global currentdir
    set types {       
        {{BMP Bitmap File}                      {.bmp}        }
        {{TIFF Image File}                      {.tif}        }
        {{All Files }                           *             }
    }
    set outfilename [tk_getSaveFile -filetypes $types -initialdir $currentdir]
    set currentdir [file dirname $outfilename]
    SaveSnapImageViewer $outfilename
}

proc SaveSnapImageViewer {outfilename} {
# Virker ikke
    if { $outfilename == "" } {return}
    if { $outfilename != "" } {
      if { [string match *.bmp $outfilename] } {
        vtkRendererSource renSrc
          renSrc SetInput [viewer GetOutput]
          renSrc WholeWindowOn
        vtkBMPWriter BMPwriter
          BMPwriter SetInput [renSrc GetOutput]
          BMPwriter SetFileName $outfilename
          BMPwriter Write
        BMPwriter Delete
        renSrc Delete
      } elseif { [string match *.tif $outfilename] } {
          vtkRendererSource renSrc
          renSrc SetInput viewer
          renSrc WholeWindowOn
        vtkTIFFWriter TIFFwriter
          TIFFwriter SetInput [renSrc GetOutput]
          TIFFwriter SetFileName $outfilename
          TIFFwriter Write
        TIFFwriter Delete
        renSrc Delete
      } else {
            puts "Can't write this file"
            return
      }
    }
}


