package require vtk
package require vtkinteraction

scan [[CurrentVolume GetOutput] GetWholeExtent] "%d %d %d %d %d %d" \
        xMin xMax yMin yMax zMin zMax

# Compute the middle of the volume - for initial slice positioning
set xMid [expr (( $xMax - $xMin ) / 2 )]
set yMid [expr (( $yMax - $yMin ) / 2 )]
set zMid [expr (( $zMax - $zMin ) / 2 )]

puts "xMin xMax yMin yMax zMin zMax $xMin $xMax $yMin $yMax $zMin $zMax"

set spacing [[CurrentVolume GetOutput] GetSpacing]
set sx [lindex $spacing 0]
set sy [lindex $spacing 1]
set sz [lindex $spacing 2]

set origin [[CurrentVolume GetOutput] GetOrigin]
set ox [lindex $origin 0]
set oy [lindex $origin 1]
set oz [lindex $origin 2]

# An outline is shown for context.
#
vtkOutlineFilter poutline
  poutline SetInput [CurrentVolume GetOutput]

vtkPolyDataMapper poutlineMapper
  poutlineMapper SetInput [poutline GetOutput]

vtkActor poutlineActor
  poutlineActor SetMapper poutlineMapper

# The shared picker enables us to use 3 planes at one time
# and gets the picking order right
#
iren SetPicker picker
vtkCellPicker ppicker
  ppicker SetTolerance 0.005
  #picker SetEndPickMethod annotatePickClosest
  #picker SetEndPickMethod  annotatePickOutlineMode
  #v42#picker SetEndPickMethod annotatePick
  picker AddObserver EndPickEvent annotatePick


# The 3 image plane widgets are used to probe the dataset.
#
vtkImagePlaneWidget planeWidgetX
  planeWidgetX DisplayTextOn
  planeWidgetX SetInput [CurrentVolume GetOutput]
  planeWidgetX SetPlaneOrientationToXAxes
  planeWidgetX SetSliceIndex $xMid
  planeWidgetX SetPicker ppicker
  planeWidgetX SetKeyPressActivationValue "x"
  planeWidgetX KeyPressActivationOn
  set prop1 [planeWidgetX GetPlaneProperty]
  $prop1 SetColor 1 0 0  

vtkImagePlaneWidget planeWidgetY
  planeWidgetY DisplayTextOn
  planeWidgetY SetInput [CurrentVolume GetOutput]
  planeWidgetY SetPlaneOrientationToYAxes
  planeWidgetY SetSliceIndex $yMid
  planeWidgetY SetPicker ppicker
  planeWidgetY SetKeyPressActivationValue "y"
  set prop2 [planeWidgetY GetPlaneProperty]
  $prop2 SetColor 0 1 0
  planeWidgetY SetLookupTable [planeWidgetX GetLookupTable]

vtkImagePlaneWidget planeWidgetZ
  planeWidgetZ DisplayTextOn
  planeWidgetZ SetInput [CurrentVolume GetOutput]
  planeWidgetZ SetPlaneOrientationToZAxes
  planeWidgetZ SetSliceIndex $zMid
  planeWidgetZ SetPicker ppicker
  planeWidgetZ SetKeyPressActivationValue "z"
  set prop3 [planeWidgetZ GetPlaneProperty]
  $prop3 SetColor 0 0 1
  planeWidgetZ SetLookupTable [planeWidgetX GetLookupTable]

#vtk42#ppicker SetEndPickMethod annotatePPick
ppicker AddObserver EndPickEvent annotatePPick

set current_widget planeWidgetZ
set mode_widget  planeWidgetZ

# Create the RenderWindow and Renderer
#
vtkRenderer ren2
vtkRenderWindow renWin2
  renWin2 AddRenderer ren2

# Add the outline actor to the renderer, set the background color and size
#
ren2 AddActor poutlineActor
renWin2 SetSize 800 800
ren2 SetBackground  0.1 0.1 0.2

# Create a cube to be shown at cursor center
#set VolumeSlicerCursorCubeWidthXfactor 0.5
#set VolumeSlicerCursorCubeWidthYfactor 0.5
#set VolumeSlicerCursorCubeWidthZfactor 0.5
#set VolumeSlicerCursorCubeOpacity 0.5
vtkCubeSource cursorCube
     cursorCube SetXLength [expr $sx * $VolumeSlicerCursorCubeWidthXfactor]
     cursorCube SetYLength [expr $sy * $VolumeSlicerCursorCubeWidthYfactor]
     cursorCube SetZLength [expr $sz * $VolumeSlicerCursorCubeWidthZfactor]
     cursorCube SetCenter 0 0 0
vtkPolyDataMapper cursorCubeMapper
     cursorCubeMapper SetInput [cursorCube GetOutput]
vtkActor cursorCubeActor 
     cursorCubeActor SetMapper cursorCubeMapper
     [cursorCubeActor GetProperty] SetColor 0 1 1 
     [cursorCubeActor GetProperty] SetOpacity $VolumeSlicerCursorCubeOpacity
     cursorCubeActor PickableOff
     cursorCubeActor VisibilityOn
     ren2 AddActor cursorCubeActor

# Create a cube to be shown at cursor center
vtkCubeSource cursorCube2
     cursorCube2 SetXLength [expr $lmDiameter * 20]
     cursorCube2 SetYLength [expr $lmDiameter * 20]
     cursorCube2 SetZLength [expr $lmDiameter * 20]
     cursorCube2 SetCenter 0 0 0
vtkPolyDataMapper cursorCubeMapper2
     cursorCubeMapper2 SetInput [cursorCube2 GetOutput]
vtkActor cursorCubeActor2 
     cursorCubeActor2 SetMapper cursorCubeMapper2
     [cursorCubeActor2 GetProperty] SetColor 1 1 1 
     [cursorCubeActor2 GetProperty] SetOpacity 1.0
     cursorCubeActor2 PickableOff
     cursorCubeActor2 VisibilityOn
     #ren2 AddActor cursorCubeActor2

set SphereWidgetPlaced 0
if {$SphereWidgetCreated} {
   puts stdout "tttttttttttttttttttttttttttttttttttttttttttt"
   ren2 AddActor PolySphereWidgetActor
   set SphereWidgetPlaced 1
}

# Create the GUI
#
##wm withdraw .
##destroy .top
toplevel .top2
wm title .top2 "Volume Slicer"
wm protocol .top2 WM_DELETE_WINDOW ::vtk::cb_exit

frame .f2
menu .top2.menubar2
.top2 config -menu .top2.menubar2
foreach mm {File Options Settings Orientation} {
  set $mm [menu .top2.menubar2.mm$mm]
  .top2.menubar2 add cascade -label $mm -menu .top2.menubar2.mm$mm -underline 0
}
$File add cascade -label Save -menu $File.sub3
$File add separator
$File add command -label Exit -command ::vtk::cb_exit -foreground red -underline 1
$Options add check -label "Toggle X Plane" -command ToggleXPlane -variable ToggleXPlaneCheck
$Options add check -label "Toggle Y Plane" -command ToggleYPlane -variable ToggleYPlaneCheck
$Options add check -label "Toggle Z Plane" -command ToggleZPlane -variable ToggleZPlaneCheck
$Options add check -label "Toggle Surface(s)" -command ToggleSurfaces -variable ToggleSurfaceCheck
$Options add check -label "Toggle Landmarks" -command ToggleLandmarks -variable ToggleLandmarksCheck
$Options add check -label "Toggle LineCurve" -command ToggleLineCurve -variable ToggleLineCurveCheck
$Options add check -label "Toggle Sphere Surface" -command ToggleSphere -variable ToggleSphereCheck
$Settings add command -label "Background Color" -command BackgroundColorVolumeSlicer
set slice_orientation 0
$Orientation add radio -label "  0 deg" -value 0 -variable slice_orientation
$Orientation add radio -label " 90 deg" -value 90 -variable slice_orientation
$Orientation add radio -label "180 deg" -value 180 -variable slice_orientation
$Orientation add radio -label "270 deg" -value 270 -variable slice_orientation
set m3 [menu $File.sub3 -tearoff 0]
$m3 add command -label "Sphere Surface" -command SaveSphereWidgetSurface
$m3 add command -label "Snapshot" -command SaveSnapshotVolumeSlicer
set ToggleSphereCheck 1
set ToggleSurfaceCheck 0
set ToggleLandmarksCheck 0
set ToggleXPlaneCheck 1
set ToggleYPlaneCheck 1
set ToggleZPlaneCheck 1

pack .f2

set popm [menu .top2.mm -tearoff 0]
set mode 2
$popm add radiobutton -label "nearest" -variable mode -value 0  \
           -command SetInterpolation
$popm add radiobutton -label "linear" -variable mode -value 1  \
           -command SetInterpolation
$popm add radiobutton -label "cubic" -variable mode -value 2  \
           -command SetInterpolation

set display_frame [frame .top2.f1]

set ctrl_buttons [frame .top2.btns]

pack $display_frame $ctrl_buttons \
        -side top -anchor n \
        -fill both -expand f

#set quit_button [button $ctrl_buttons.btn1  \
#        -text "Quit" \
#        -command  ::vtk::cb_exit]

set capture_button [button $ctrl_buttons.btn2  \
        -text "Tif" \
        -command CaptureImage]

set x_button [button $ctrl_buttons.btn3  \
        -text "x" \
        -command AlignXaxis]

set y_button [button $ctrl_buttons.btn4  \
        -text "y" \
        -command AlignYaxis]

set z_button [button $ctrl_buttons.btn5  \
        -text "z" \
        -command AlignZaxis]

set goto_button [button $ctrl_buttons.btn6  \
        -text "GOTO CURSOR" \
        -command GotoPosition]

set goto_landmark_button [button $ctrl_buttons.btn7  \
        -text "GOTO LANDMARK" \
        -command GotoLandmarkPosition]

set set_landmark_button [button $ctrl_buttons.btn8  \
        -text "SET LANDMARK" \
        -command SetLandmark]



set last_btn -1
bind $x_button <Button-3> "set last_btn 0; configMenu; $popm post %X %Y"
bind $y_button <Button-3> "set last_btn 1; configMenu; $popm post %X %Y"
bind $z_button <Button-3> "set last_btn 2; configMenu; $popm post %X %Y"

# Share the popup menu among buttons, keeping
# track of associated widget's interpolation mode
#
proc configMenu { } {
  global last_btn popm mode mode_widget
  if { $last_btn == 0 } {
    set mode_widget planeWidgetX
  } elseif { $last_btn == 1 } {
    set mode_widget planeWidgetY
  } else {
    set mode_widget planeWidgetZ
  }
  set mode [$mode_widget GetResliceInterpolate]    
  $popm entryconfigure $last_btn -variable mode   
}

pack $x_button $y_button $z_button $goto_button $goto_landmark_button $set_landmark_button \
        -side left \
        -expand t -fill both

# Create the render widget
#
set renderer_frame [frame $display_frame.rFm]

pack $renderer_frame \
        -padx 3 -pady 3 \
        -side left -anchor n \
        -fill both -expand f

set render_widget [vtkTkRenderWidget $renderer_frame.r \
        -width $VolumeSlicerWindowSizeX \
        -height $VolumeSlicerWindowSizeY \
        -rw renWin2]

pack $render_widget $display_frame  \
        -side top -anchor n \
        -fill both -expand f

# Add a slice scale to browse the current slice stack
#

set imslice_number [$current_widget GetSliceIndex]

scale .top2.slice \
        -from $zMin \
        -to $zMax \
        -orient horizontal \
        -command SetImSlice \
        -variable imslice_number \
        -label "Slice"

pack .top2.slice \
        -fill x -expand f

proc SetImSlice {slice} {
  global current_widget 
  global imslice_number
  $current_widget SetSliceIndex $slice
  ren2 ResetCameraClippingRange
  renWin2 Render
}

::vtk::bind_tk_render_widget $render_widget
# Set the interactor for the widgets
#
set iact [[$render_widget GetRenderWindow] GetInteractor]

planeWidgetX SetInteractor $iact
planeWidgetX On
planeWidgetY SetInteractor $iact
planeWidgetY On
planeWidgetZ SetInteractor $iact
planeWidgetZ On

# Create an initial interesting view
#
set cam1 [ren2 GetActiveCamera]
$cam1 Elevation 110
$cam1 SetViewUp 0 0 -1
$cam1 Azimuth 45
ren2 ResetCameraClippingRange

[ren2 GetActiveCamera] ParallelProjectionOn

# Modify some bindings, use the interactor style 'switch'
#
set iren2 [renWin2 GetInteractor]
set istyle2 [vtkInteractorStyleSwitch istyleswitch2]
$iren2 SetInteractorStyle $istyle2
$istyle2 SetCurrentStyleToTrackballCamera

# Render it
#
$render_widget Render


# Supporting procedures
#

proc RollSlicerCamera {} {
   set camera [ren2 GetActiveCamera]
   set roll [$camera GetRoll]
   $camera SetRoll [expr $roll + 90]
   ##renWin2 Render
}

# Align the camera so that it faces the desired widget
#
proc AlignCamera { } {
  global ox oy oz sx sy sz xMax xMin yMax yMin zMax zMin imslice_number
  global current_widget slice_orientation
  set cx [expr $ox + (0.5*($xMax - $xMin))*$sx]
  set cy [expr $oy + (0.5*($yMax - $yMin))*$sy]
  set cz [expr $oy + (0.5*($zMax - $zMin))*$sz]
  set vx 0
  set vy 0
  set vz 0
  set nx 0
  set ny 0
  set nz 0
  set iaxis [$current_widget GetPlaneOrientation]
  if { $iaxis == 0 } {
    set vz -1
    set nx [expr $ox + $xMax*$sx]
    set cx [expr $ox + $imslice_number*$sx]
  }  elseif  { $iaxis == 1 } {
    set vz -1
    set ny [expr $oy + $yMax*$sy]
    set cy [expr $oy + $imslice_number*$sy]
  } else {
    set vy 1
    set nz [expr $oz + $zMax*$sz]
    set cz [expr $oz + $imslice_number*$sz]
  }
  set px [expr $cx + $nx*2]
  set py [expr $cy + $ny*2]
  set pz [expr $cz + $nz*3]

  set camera [ ren2 GetActiveCamera ]
  $camera SetViewUp $vx $vy $vz
  $camera SetFocalPoint $cx $cy $cz
  $camera SetPosition $px $py $pz
  $camera OrthogonalizeViewUp
  ren2 ResetCameraClippingRange 
  if {$slice_orientation == 90} {
    RollSlicerCamera
  } elseif {$slice_orientation == 180} {
    RollSlicerCamera
    RollSlicerCamera
  } elseif {$slice_orientation == 270} {
    RollSlicerCamera
    RollSlicerCamera
    RollSlicerCamera
  } else {
  }
  renWin2 Render
}

# Capture the display and place in a tiff
#
proc CaptureImage { } {
  vtkWindowToImageFilter w2i
  vtkTIFFWriter writer

  w2i SetInput renWin2
  w2i Update
  writer SetInput [w2i GetOutput]
  writer SetFileName image.tif
  renWin2 Render
  writer Write

  writer Delete
  w2i Delete
}

proc SaveSphereWidgetSurface {} {
   global SphereWidgetCreated
   # Save a representation of the sphere widget (typically used as an eye).
   # We copy the sphere widget representation into PolyDataObject and write it.
   if {! $SphereWidgetCreated} {
      notify_error "No Sphere widget to save!"
      return -1
   }  

   vtkPolyData PolyDataObject 

      PolyDataObject CopyStructure [PolySphereWidget GetOutput]
      [PolyDataObject GetPointData] PassData [[PolySphereWidget GetOutput] GetPointData]
      PolyDataObject Modified
      PolyDataObject Update

     ##PolyDataObject SetInput [PolySphereWidget GetOutput]
   SavePolyDataObject
   PolyDataObject Delete
}



proc SaveSnapVolumeSlicer {outfilename} {
    if { $outfilename == "" } {return}
    if { $outfilename != "" } {
      if { [string match *.bmp $outfilename] } {
        vtkRendererSource renSrc
          renSrc SetInput ren2
          renSrc WholeWindowOn
        vtkBMPWriter BMPwriter
          BMPwriter SetInput [renSrc GetOutput]
          BMPwriter SetFileName $outfilename
          BMPwriter Write
        BMPwriter Delete
        renSrc Delete
      } elseif { [string match *.tif $outfilename] } {
          vtkRendererSource renSrc
          renSrc SetInput ren2
          renSrc WholeWindowOn
        vtkTIFFWriter TIFFwriter
          TIFFwriter SetInput [renSrc GetOutput]
          TIFFwriter SetFileName $outfilename
          TIFFwriter Write
        TIFFwriter Delete
        renSrc Delete
      } else {
            puts "Can't write this file"
            return
      }
    }
}

# Procedure to save snapshot as a file.
proc SaveSnapshotVolumeSlicer {} {
    global currentdir
    set types {       
        {{BMP Bitmap File}                      {.bmp}        }
        {{TIFF Image File}                      {.tif}        }
        {{All Files }                           *             }
    }
    set outfilename [tk_getSaveFile -filetypes $types -initialdir $currentdir]
    set currentdir [file dirname $outfilename]
    SaveSnapVolumeSlicer $outfilename
}

proc ToggleLineCurve {} {
  # To be executed when "Toggle LineCurve" is toggled on or off.
  global LineCrvFileHasBeenRead ToggleLineCurveCheck oactorNum
  global lineCrvPoints linesegmentNum pNum
  global crv_flag 
  global lineCrvPointsRed lineCrvPointsGreen lineCrvPointsBlue lineCrvPointsOpacity
  global currentdir
  global lmDiameter tubeDiameterFactor
  global additional_crv_flag LineCrvFileHasBeenRead
  global tubeColorRed tubeColorGreen tubeColorBlue tubeColorOpacity
     
  if {! $LineCrvFileHasBeenRead} {
     return -1
  }
  if {$ToggleLineCurveCheck} {
     ShowLineCrvInVolumeSlicer
  } else {
     puts stdout "do nothing for now..."
  } 
  renWin2 Render
}


proc ToggleSurface {} {
  # To be executed when "Toggle Surface" is toggled on or off.
  global SurfaceFileHasBeenRead ToggleSurfaceCheck oactorNum
     
  if {! $SurfaceFileHasBeenRead} {
     return -1
  }
  if {$ToggleSurfaceCheck} {
     ren2 AddActor oactor0
  } else {
     ren2 RemoveActor oactor0  
  } 
  renWin2 Render
}

proc ToggleSurfaces {} {
  # To be executed when "Toggle Surface" is toggled on or off (toggles all surfaces, also those loaded with "additional")
  global SurfaceFileHasBeenRead ToggleSurfaceCheck oactorNum
     
  if {! $SurfaceFileHasBeenRead} {
     return -1
  }
  if {$ToggleSurfaceCheck} {
     for {set i 0} {$i <= $oactorNum} {incr i 1} { 
       ren2 AddActor oactor$i
       puts stdout "AddActor oactor $oactorNum $i"
     }
  } else {
     for {set i 0} {$i <= $oactorNum} {incr i 1} { 
       ren2 RemoveActor oactor$i
       puts stdout "RemoveActor oactor $oactorNum $i"

     }  
  } 
  renWin2 Render
}


proc ToggleLandmarks {} {
  # To be executed when "Toggle Landmarks" is toggled on or off.
  # We go through all landmark actors and turn them on or off.
  global LandmarkFileHasBeenRead ToggleLandmarksCheck 
  global setarr maxNlandmarks maxlmIndex
     
  if {$maxlmIndex == 0} {
     return -1
  }
  if {$ToggleLandmarksCheck} { 
     for {set i 0} {$i < $maxNlandmarks} {incr i 1} {
       if {$setarr($i) == 1} {
         ren2 AddActor lmsphereActor$i
       }
     }
  } else {
     for {set i 0} {$i < $maxNlandmarks} {incr i 1} {
       if {$setarr($i) == 1} {
         ren2 RemoveActor lmsphereActor$i
       }
     }
  }
  renWin2 Render
}


proc ToggleSphere {} {
  # To be executed when "Toggle Sphere" is toggled on or off.
  global SurfaceFileHasBeenRead ToggleSphereCheck SphereWidgetCreated
     
  if {! $SphereWidgetCreated} {
     return -1
  }
  if {$ToggleSphereCheck} {
     ren2 AddActor PolySphereWidgetActor
  } else {
     ren2 RemoveActor PolySphereWidgetActor
  } 
  renWin2 Render
}

proc ToggleXPlane {} {
  global ToggleXPlaneCheck
  set proptx [planeWidgetX GetTexturePlaneProperty]
  set proppx [planeWidgetX GetPlaneProperty]
  if {$ToggleXPlaneCheck} {
     $proptx SetOpacity 1.0
  } else {
     $proptx SetOpacity 0.0
  }
  renWin2 Render
}

proc ToggleYPlane {} {
  global ToggleYPlaneCheck
  set propty [planeWidgetY GetTexturePlaneProperty]
  set proppy [planeWidgetY GetPlaneProperty]
  if {$ToggleYPlaneCheck} {
     $propty SetOpacity 1.0
  } else {
     $propty SetOpacity 0.0
  }
  renWin2 Render
}

proc ToggleZPlane {} {
  global ToggleZPlaneCheck
  set proptz [planeWidgetZ GetTexturePlaneProperty]
  set proppz [planeWidgetZ GetPlaneProperty]
  if {$ToggleZPlaneCheck} {
     $proptz SetOpacity 1.0
    } else {
     $proptz SetOpacity 0.0
  }
  renWin2 Render
}



proc GotoPosition { } {
   # Goto current cursor position.
   global xp yp zp
   planeWidgetX SetSlicePosition $xp
   planeWidgetY SetSlicePosition $yp
   planeWidgetZ SetSlicePosition $zp
   cursorCube SetCenter $xp $yp $zp
   renWin2 Render
}

proc GotoLandmarkPosition { } {
   # Goto current landmark position.
   global xp yp zp
   global lmCoord lmIndex
   set xyz [lmCoord GetPoint $lmIndex]
   set xp [lindex $xyz 0] 
   set yp [lindex $xyz 1] 
   set zp [lindex $xyz 2] 
   planeWidgetX SetSlicePosition $xp
   planeWidgetY SetSlicePosition $yp
   planeWidgetZ SetSlicePosition $zp
   cursorCube SetCenter $xp $yp $zp
   renWin2 Render
}


proc SetLandmark { } {
   global xp yp zp
   global lmIndex lmCoord setarr setbuttonvalue SplineCheck
        lmsphere$lmIndex SetCenter $xp $yp $zp
        lmsphereActor$lmIndex VisibilityOn
        lmCoord InsertPoint $lmIndex $xp $yp $zp
        set setarr($lmIndex) 1
        set setbuttonvalue 1  
        #SetIndex -1
        #SetSetLandmark 
        SetSetPos 
        if {$SplineCheck == 1} {
          UpdateSpline
        }
   renWin Render
}



# Align the widget back into orthonormal position,
# set the slider to reflect the widget's position,
# call AlignCamera to set the camera facing the widget
#
proc AlignXaxis { } {
  global xMax xMin current_widget imslice_number
  set po [ planeWidgetX GetPlaneOrientation ]
  if { $po == 3 } {
    planeWidgetX SetPlaneOrientationToXAxes
    set imslice_number [expr ($xMax - $xMin)/2]
    planeWidgetX SetSliceIndex $imslice_number
  } else {
    set imslice_number [planeWidgetX GetSliceIndex]
  }
  set current_widget planeWidgetX
  .top2.slice config -from $xMin -to $xMax 
  .top2.slice set $imslice_number
  AlignCamera
}

proc AlignYaxis { } {
  global yMin yMax current_widget imslice_number
  set po [ planeWidgetY GetPlaneOrientation ]
  if { $po == 3 } {
    planeWidgetY SetPlaneOrientationToYAxes
    set imslice_number [expr ($yMax - $yMin)/2]
    planeWidgetY SetSliceIndex $imslice_number
  } else {
    set imslice_number [planeWidgetY GetSliceIndex]
  }
  set current_widget planeWidgetY
  .top2.slice config -from $yMin -to $yMax
  .top2.slice set $imslice_number
  AlignCamera
}

proc AlignZaxis { } {
  global zMin zMax current_widget imslice_number
  set po [ planeWidgetZ GetPlaneOrientation ]
  if { $po == 3 } {
    planeWidgetZ SetPlaneOrientationToZAxes
    set imslice_number [expr ($zMax - $zMin)/2]
    planeWidgetZ SetSliceIndex $imslice_number
  } else {
    set imslice_number [planeWidgetZ GetSliceIndex]
  }
  set current_widget planeWidgetZ
  .top2.slice config -from $zMin -to $zMax 
  .top2.slice set $imslice_number
  AlignCamera
}

# Set the widget's reslice interpolation mode
# to the corresponding popup menu choice
#
proc SetInterpolation { } {
  global mode_widget mode 
  if { $mode == 0 } {
    $mode_widget TextureInterpolateOff
  } else {
    $mode_widget TextureInterpolateOn
  }
  $mode_widget SetResliceInterpolate $mode
  renWin2 Render
}

proc annotatePPick {} {
    global lmCoord lmIndex setbuttonvalue setarr CurrentPos CurrentId CurrentVal
    global aSplineX aSplineY aSplineZ SplineCheck xp yp zp
    global colors existcolorfile
    global lmCellIdArr
    global SphereWidgetCreated SphereWidgetPlaced
    if { [ppicker GetCellId] < 0 } {
        #lmsphereActor$lmIndex VisibilityOff
        #set pickId -1
    } else {
 
	#set selPt [ppicker GetSelectionPoint]
        #set pickId [picker GetCellId]
        #set lmCellIdArr($lmIndex) $pickId
        #set CurrentId [format "Id: %10d" $pickId]
        #set CurrentVal " "
        #if {$existcolorfile == 1} {
        #    set pickVal [colors GetScalar $pickId]
        #    set CurrentVal [format "Value: %18.6f" $pickVal]
        #}
	#set x [lindex $selPt 0] 
	#set y [lindex $selPt 1]
	set pickPos [ppicker GetPickPosition]
	set xp [lindex $pickPos 0] 
	set yp [lindex $pickPos 1]
	set zp [lindex $pickPos 2]

      #planeWidgetX SetSlicePosition $xp
      #planeWidgetY SetSlicePosition $yp
      #planeWidgetZ SetSlicePosition $zp

        cursorCube SetCenter $xp $yp $zp
        cursorCube2 SetCenter $xp $yp $zp

        
    }
    
    if {$SphereWidgetCreated} {
       # Let the sphere widget follow the picked position.
       if {! $SphereWidgetPlaced} {
           ren2 AddActor PolySphereWidgetActor
           set SphereWidgetPlaced 1
       }
       sphereWidget SetCenter $xp $yp $zp
       PolySphereWidget SetCenter $xp $yp $zp
       PolySphereWidget SetRadius [sphereWidget GetRadius]
       renWin2 Render
    }
    renWin Render
}


proc BackgroundColorVolumeSlicer {} {

    set background [ren2 GetBackground]
    .back2.f1.l.r set [expr [lindex $background 0] * 255.0]
    .back2.f1.l.g set [expr [lindex $background 1] * 255.0]
    .back2.f1.l.b set [expr [lindex $background 2] * 255.0]
    wm deiconify .back2
}

proc CloseBackgroundVolumeSlicer {} {
    wm withdraw .back2
}

toplevel .back2
wm withdraw .back2
wm title .back2 "Volume Slicer Background Color"
wm protocol .back2 WM_DELETE_WINDOW {wm withdraw .back2}
frame .back2.f1

frame .back2.f1.l  -relief raised -borderwidth 3
scale .back2.f1.l.r -from 255 -to 0 -orient vertical -background #f00 \
	-command SetColor
scale .back2.f1.l.g -from 255 -to 0 -orient vertical -background #0f0 \
	-command SetColor
scale .back2.f1.l.b -from 255 -to 0 -orient vertical -background #00f \
	-command SetColor
pack .back2.f1.l.r .back2.f1.l.g .back2.f1.l.b -side left -fill both

frame .back2.f1.m -relief raised -borderwidth 3
label .back2.f1.m.sample -highlightthickness 0 -text "  Background Color  "
pack .back2.f1.m.sample -fill both -expand 1

frame .back2.f1.r -relief raised -borderwidth 3
set colwheel "ColorWheel.ppm"
set colwheel $landmarker_home$colwheel
image create photo ColorWheel -file $colwheel
label .back2.f1.r.wheel -image ColorWheel -highlightthickness 0
bind .back2.f1.r.wheel <Button-1> {
    scan [ColorWheel get %x %y] "%%f %%f %%f" r g b
    .back2.f1.l.r set $r
    .back2.f1.l.g set $g
    .back2.f1.l.b set $b
}
pack .back2.f1.r.wheel -fill both
pack .back2.f1.l .back2.f1.m .back2.f1.r -side left -expand 1 -fill both

frame .back2.fb
button .back2.fb.apply -text Apply -command ApplyBackgroundVolumeSlicer
button .back2.fb.defult -text Default -command ApplyDefaultVolumeSlicer
button .back2.fb.black -text Black -command ApplyBlackVolumeSlicer
button .back2.fb.white -text White -command ApplyWhiteVolumeSlicer
button .back2.fb.cancel -text Done -command CloseBackgroundVolumeSlicer
pack .back2.fb.apply .back2.fb.defult .back2.fb.black .back2.fb.white .back2.fb.cancel -side left -expand 1 -fill x
pack .back2.f1 .back2.fb -side top -fill both -expand 1

proc SetColor {value} {
    set color [format #%02x%02x%02x [.back2.f1.l.r get] [.back2.f1.l.g get]\
	    [.back2.f1.l.b get]]
    .back2.f1.m.sample config -background $color
}

proc ApplyBackgroundVolumeSlicer {} {
    ren2 SetBackground [expr [.back2.f1.l.r get]/255.0] \
	    [expr [.back2.f1.l.g get]/255.0] \
	    [expr [.back2.f1.l.b get]/255.0]
    
    renWin2 Render
}

proc ApplyDefaultVolumeSlicer {} {
    global renWin
    ren2 SetBackground 0.1 0.1 0.2

    set background [ren1 GetBackground]
    .back2.f1.l.r set [expr [lindex $background 0] * 255.0]
    .back2.f1.l.g set [expr [lindex $background 1] * 255.0]
    .back2.f1.l.b set [expr [lindex $background 2] * 255.0]
    renWin2 Render
}
proc ApplyWhiteVolumeSlicer {} {
    ren2 SetBackground 1.0 1.0 1.0
    set background [ren1 GetBackground]
    .back2.f1.l.r set [expr [lindex $background 0] * 255.0]
    .back2.f1.l.g set [expr [lindex $background 1] * 255.0]
    .back2.f1.l.b set [expr [lindex $background 2] * 255.0]
    renWin2 Render
}
proc ApplyBlackVolumeSlicer {} {
    ren2 SetBackground 0.0 0.0 0.0
    set background [ren1 GetBackground]
    .back2.f1.l.r set [expr [lindex $background 0] * 255.0]
    .back2.f1.l.g set [expr [lindex $background 1] * 255.0]
    .back2.f1.l.b set [expr [lindex $background 2] * 255.0]
    renWin2 Render
}

