###################################################################################
# File: general_settings.tcl
# Parameter settings for face_analysis programs.
# This file contains parameters that are not likely to change very often.
###################################################################################

###################################################################################
## GENERAL PARAMETERS

# TEMPLATE contains the name of the input template (must be located in DATA_DIRECTORY_TEMPLATE).
set TEMPLATE "face_template.vtk"

# DATA_DIR_TEMPLATE contains path (relative to ROOT_DIR) to the input patient data of the source template (must end with a /).
# Note that even on a Windows computer the directory delimeter should be / (slash) and not backslash (\). 
set DATA_DIR_TEMPLATE "face_asymmetry/"

# TEMPLATE_NTRIANGLES is number of triangles in the template (needed only for the asymmetry_ho_xyz and asymmetry_ho_xyz_sizecorr methods).
set TEMPLATE_NTRIANGLES 24672

# REFERENCE_SIZE is a reference centroid size (needed for asymmetry_ho_xyz_sizecorr method).
##set REFERENCE_SIZE 73.377

# mask is name of the .clr file where the ROI region is defined (e.g. helmet region or face).
# It's location is in DATA_DIR_TEMPLATE
set roi_mask all_match_average_template_facemask2_clr.clr

# color_mask_only should be set if you only want to color results in the roi region.
set color_roi_mask_only 1

# Set EXIT_WHEN_DONE to 1 if you wish the program to exit landmarker after execution. 
# Should be set to 1 when you run several tcl-scripts from batch using a .cmd file.
set EXIT_WHEN_DONE 1

# If CLEANUP is 1 then temporary files will be deleted from the output directory.
# If CLEANUP is 0 then the temporary files will be left in the output directory, for debugging purposes etc.
set CLEANUP 0

###################################################################################


###################################################################################
## PARAMETERS RELATED TO ORIENTATION (ORIENT FLAG IN THE TASKLIST)

# APPLICATION_ORIENT contains name of the .tcl script that will carry out orientation of the scan to the template.
# Note that the path is relative to ROOT_DIR and with / as delimeter in the path.
#set APPLICATION_ORIENT "general/source_code/rigid_orient_to_template.tcl"
##set APPLICATION_ORIENT "general/source_code/msp_fhp_orient.tcl"

# Here we define the list of landmarks used for the rigid body transform of scan to template.
# You may change the landmarks used for this transformation here.
set rigid_trafo_list {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17}
###################################################################################


###################################################################################
## PARAMETERS RELATED TO TEMPLATE MATCHING (MATCH FLAG IN THE TASKLIST)

# APPLICATION_MATCHING contains name of the .tcl script that will carry out the template matching.
# Note that the path is relative to ROOT_DIR and with / as delimeter in the path.
# set APPLICATION_MATCHING "general/source_code/template_match_no_eyebrow_correct_with_scaling.tcl"

# template_match_eyebrow_correct.tcl needs to know the landmark number of the eyebrows. Give rigth first, left as second.
set eyebrows_right_left {10 11}

# Here we define the list of landmarks used for the TPS transform of template to scan.
# You may change the landmarks used for this transformation here.
##set tps_trafo_list {0 1 2 4 5 6 7 8 9 10 11 12 13 14 16 17 18 19}
set tps_trafo_list {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19}

# rightear_leftear_nasion_list is a list of these three landmarks' numbers.
set rightear_leftear_nasion_list {0 1 2}

# x/y/z_scale_landmarks need each to be set to two landmarks used for scaling of the template before matching. (Landmarks must exist!)
# In Lara's study AFTER ORIENT: x=horiz, y=sagit, z=vert; 0=right_ear, 1=left_ear, 2=nasion, 3=nose_tip, 14=upper_lip, 17=chin
set x_scale_landmarks {0 1}
set y_scale_landmarks {0 2}
set z_scale_landmarks {2 14}

###################################################################################


###################################################################################
## PARAMETERS RELATED TO ASYMMETRY CALCULATION (ASY_HO_XYZ FLAG IN THE TASKLIST)

# APPLICATION_ASYMMETRY_HO_XYZ contains name of the .tcl script that will compute asymmetry by Hildur's method.
# Note that the path is relative to ROOT_DIR and with / as delimeter in the path.
##set APPLICATION_ASYMMETRY_HO_XYZ "general/source_code/asymmetry_ho_xyz_single.tcl"
##set APPLICATION_ASYMMETRY_HO_XYZ "general/source_code/asymmetry_ho_xyz_single_sizecorr.tcl"

# COLOR_RANGE_ASY_HO_XYZ_MIN is the minimum value of the color range for the Hildur asymmetry computation.
set COLOR_RANGE_ASY_HO_XYZ_MIN -6.0

# COLOR_RANGE_ASY_HO_XYZ_MAX is the minimum value of the color range for the Hildur asymmetry computation.
set COLOR_RANGE_ASY_HO_XYZ_MAX 6.0

# COLOR_RANGE_ASY_HO_XYZ_AVERAGE_MIN is the minimum value of the color range for the Hildur asymmetry computation IN THE AVERAGE.
set COLOR_RANGE_ASY_HO_XYZ_AVERAGE_MIN -2.0

# COLOR_RANGE_ASY_HO_XYZ_AVERAGE_MAX is the maximum value of the color range for the Hildur asymmetry computation IN THE AVERAGE.
set COLOR_RANGE_ASY_HO_XYZ_AVERAGE_MAX 2.0